/* --------------------------------------------------------------------------------
 #
 #	4DPlugin.c
 #	source generated by 4D Plugin Wizard
 #	Project : QTPlay
 #	author : Thomas Maul, 4D Germany
 #	03.07.2005
 #
 #  Modified by Louis Thoumin, 4D S.A. on 25.03.2006
 #
 # --------------------------------------------------------------------------------*/



#include "QTPlayextern.h"
#include "QTFile.h"
#include "4DPlugin.h"

void PluginMain( long selector, PA_PluginParameters params )
{
	switch( selector )
	{
		case kInitPlugin :
			InitPlugin();
			break;

		case kDeinitPlugin :
			DeinitPlugin();
			break;

// --- QTSimple

		case eCmd_QTPlayMovie :
			QTPlayMovie( params );
			break;

		case eCmd_QTPlayStop :
			QTPlayStop( );
			break;

// --- QTControl

		case eCmd_QTSetMovie :
			QTSetMovie( params );
			break;


		case eCmd_QTSetControl :
			QTSetControl( params );
			break;

		case eCmd_QTGetControl :
			QTGetControl( params );
			break;

// --- External areas

		case eCmd_QTContainer :
			QTContainer( params );
			break;

		case eCmd_QTGetTag :
			QTGetTag( params );
			break;

// 25.3.2006 Louis Thoumin - Begin
// Offscreen area. Don't need to be in a form. 
		case eCMD_QTNewOffscreenArea:
			QTNewOffscreenArea ( params );
			break;

		case eCMD_QTDisposeMovieFromOffscreenArea:
			QTDisposeMovie ( params );
			break;

		case eCMD_QTDeleteOffscreenArea:
			QTDeleteOffscreenArea ( params );
			break;

// 25.3.2006 Louis Thoumin - End
		
		case eCMD_GetScreenshot:
			QTScreenshot( params );
			break;
	}
}

void InitPlugin()
{
	QT_Init();

	PA_ReturnNullTerminatedText(1); // 25.3.2006 Louis Thoumin
}

void DeinitPlugin()
{
	QT_DeInit();
}

// ----------------------------------- QTSimple -----------------------------------


void QTPlayMovie( PA_PluginParameters params )
{
	long Pfad_len;
	char *Pfad;
	long returnValue = 0;

	Pfad_len = PA_GetTextParameter( params, 1, 0L );
	Pfad = (char*)malloc(Pfad_len);
	if ( Pfad != NULL )   {
		Pfad_len = PA_GetTextParameter( params, 1, Pfad );
		returnValue = PlayFileWithQuickTime(Pfad, Pfad_len);
		free ( Pfad );
	}
	else
		returnValue = -108;
	PA_ReturnLong( params, returnValue );
}

void QTPlayStop( void )
{
	// --- write the code of QTPlayStop here...
	StopPlayFileWithQuickTime();
}

// ----------------------------------- QTControl ----------------------------------


void QTSetMovie( PA_PluginParameters params )
{
	long returnValue = 0;

	returnValue = QT_SetMovie(params);
	PA_ReturnLong( params, returnValue );
}

void QTSetControl( PA_PluginParameters params )
{
	long returnValue=0, command, value;
	void *Area;

	Area = (void *) PA_GetLongParameter( params, 1 );
	command = PA_GetLongParameter( params, 2 );
	value = PA_GetLongParameter( params, 3 );

	returnValue = QT_SetControl(Area, command, value);

	PA_ReturnLong( params, returnValue );
}

void QTGetControl( PA_PluginParameters params )
{
	long returnValue=0, command, value=0;
	void *Area;

	Area = (void *) PA_GetLongParameter( params, 1 );
	command = PA_GetLongParameter( params, 2 );

	returnValue = QT_GetControl(Area, command, &value);
	PA_SetLongParameter(params, 3, value);

	PA_ReturnLong( params, returnValue );
}

void QTGetTag( PA_PluginParameters params )
{
	long returnValue=0;
	void *Area;
	PA_Variable Tag , Content;


	Area = (void *) PA_GetLongParameter( params, 1 );
	Tag = PA_GetVariableParameter( params, 2 );
	Content = PA_GetVariableParameter( params, 3);

	returnValue = QT_GetID3Tags(Area, &Tag, &Content);
	if (returnValue == 0)
		{
		PA_SetVariableParameter( params, 2, Tag, 0 );
		PA_SetVariableParameter( params, 3, Content, 0 );
		}

	PA_ReturnLong( params, returnValue );
}

void QTScreenshot( PA_PluginParameters params )
{
	long returnValue=0;
	void *Area;
	PA_Handle PictureShot;
	PA_PictureInfo   info = {0, 0, 0};

	Area = (void *) PA_GetLongParameter( params, 1 );
 
	returnValue = QT_GetScreenshot(Area, &PictureShot);
	if (returnValue == 0)
		PA_SetPictureHandleParameter( params, 2, PictureShot, info );

	PA_ReturnLong( params, returnValue );
}

// -------------------------------- External Areas --------------------------------


void QTContainer( PA_PluginParameters params )
{
	PA_AreaEvent event;
//	PA_PluginProperties props;
	void * privateData;

#if VERSIONWIN
	HDC         hdc;
	HBRUSH      hbrush;
	RECT         rect;
	HGDIOBJ   old;
	HFONT hf;
	char szTitle[] = "QuickTime Container v1.2 - Tech Note 2005, Thomas Maul, 4D Germany";
#else
	char szTitle[] = "\pQuickTime Container v1.2 - Tech Note 2005, Thomas Maul, 4D Germany";

#endif	
	PA_Rect   areaRect;



	event = PA_GetAreaEvent( params );
	
	switch ( event )
	{
	// --- Initialisation
		case eAE_InitAdvancedProperties :
			QTArea_PreInit( params , kIsExternalArea );  //  bug?? ACI0026601
			//  does not work...   
			//PA_DontTakeEvent( params );  // sho eAE_InitArea will still be called.
		
			break;

		case eAE_InitArea :
			QTArea_Init( params , kIsExternalArea );
			//privateData = (void*) QTArea_Init(); 
			//PA_SetAreaReference( params, privateData );		
			break;

	// --- Deinitialisation
		case eAE_Deinit :
			privateData = PA_GetAreaReference( params );
			QTArea_DeInit(privateData);
			break;

		case eAE_EndExecutionCycle:
			QTArea_Execution(params);
			break;

	// --- Update in layout editor
		case eAE_DesignUpdate :
			areaRect = PA_GetAreaRect( params );
			#if VERSIONWIN
				hdc = (HDC) PA_GetUpdateHDC();
				MyPA_RectToRECT(areaRect, &rect);
				hbrush = CreateSolidBrush( 0 );
				old = SelectObject( hdc, hbrush );
				//FrameRect( hdc, &rect, hbrush );
				FillRect(hdc, &rect, GetStockObject(WHITE_BRUSH));
				SelectObject( hdc, old );
				hf = GetStockObject(DEFAULT_GUI_FONT);
				old = SelectObject( hdc, hf );
				rect.top += 20;
				rect.left +=5;
				TextOut(hdc, rect.left, rect.top, szTitle, strlen(szTitle));
				SelectObject( hdc, old ); 
			#else 
				MoveTo(areaRect.fLeft+5 , areaRect.fTop + 20);
				DrawString(szTitle);
			#endif
			PA_CustomizeDesignMode( params );
			break;

	// --- Handle Drop of 4D objects on the area
		case eAE_AllowDrop :
			// dropInfo = PA_GetDragAndDropInfo( params );
			// Accept or refuse drop of this object on area
			// PA_AllowDrop( params, 1 );
				PA_DontTakeEvent( params );
			break;

		case eAE_Drag :
			//PA_GetDragPositions( params, &areaRect, &x, &y );
			// Handle customized dragging feedback here
			//PA_CustomizeDragOver( params );
				PA_DontTakeEvent( params );
			break;

		case eAE_Drop :
			//dropInfo = PA_GetDragAndDropInfo( params );
			// Handle drop of object here
				PA_DontTakeEvent( params );
			break;

	// --- Key down event
		case eAE_KeyDown :
			//key = PA_GetKey( params );
			// Handle key down event here
				PA_DontTakeEvent( params );
			break;

	// --- Mouse down event
		case eAE_MouseDown :
			QT_DoClick(params);
			break;

		case eAE_MouseWheel :
			// to be called when AreaEvent == eAE_MouseWheel
			// short PA_GetMouseWheelIncrement( PA_PluginParameters params )
			break;

	// --- Update and Scroll event
		case eAE_Update :
			QT_DoMovieJob(params);
		break;
			
		case eAE_Scroll :
			QT_DoScroll(params);
				break;

	// --- Select event
		case eAE_Select :
			PA_AcceptSelect( params, 1 );
			break;

	// --- Deselect event
		case eAE_Deselect :
			PA_AcceptDeselect( params, 1 );
			break;

		case eAE_PageChange:
			QT_PageChange(params);
			break;

		case eAE_Idle :
			QT_DoIdle(params);
			break;

#if __NextVersion__      
		case eAE_AreAdvancedPropertiesEditable :
			PA_SetAdvancedPropertiesEditable(params, 1);
         break;

   // We edit the properties
      case eAE_EditAdvancedProperties :
         DesignEditButton(params);
         break;

   // 4D has finish with the properties and has stored them with the form
   // we can now dispose of it.
      case eAE_DisposeAdvancedProperties :
		myptr = PA_GetAdvancedPropertiesToDispose (params);
		if (myptr) free (myptr);
         break;
#endif
	  
	  default  :
			PA_DontTakeEvent( params );
			break;

	}
}


