/*
 *  QTPlay.cpp
 *  4D Plugin
 *
 *  Created by Thomas Maul on 22.05.05.
 *  Modified by Louis Thoumin on 25.03.2006, Offscreenarea -> v1.1
 *  Read Tags, Get Screenshot, redraw optimization -> v1.2
 *
 *  Copyright 2005-2006 4D GmbH/4D S.A. All rights reserved.
 *
 */


#include "QTPlayextern.h"
#include "QTPlay.h"
#include "QTFile.h"

static short ContinuePlayMove;
static OSErr QTInitError = 0;

#if VERSIONWIN
	//static WINDOWPOS			gOldWindowPos = {nil, nil, 0, 0, 0, 0, 0};
	#define strlcpy(dest, source, size) StringCchCopyA(dest, size, source)
#else
	#ifndef min
		#define min(a,b) (((a) < (b)) ? (a) : (b))
	#endif
#endif

long PlayFileWithQuickTime (char * pfad, long len)
{
	FSSpec		myReply;
	short		myRefNum;
	OSErr		myErr=0;
	Movie		myMovie;
	
	ContinuePlayMove = 1;
	myErr = QTInitError;
	
	if(!myErr)
		{
		myErr = (OSErr) build_FSSpecFrompath(pfad, &myReply);
	
		if(!myErr)
			{
			myErr = OpenMovieFile(&myReply, &myRefNum, fsRdPerm);
			if(!myErr)
				{
				NewMovieFromFile(&myMovie, myRefNum, NULL, (StringPtr)NULL, newMovieActive, NULL);
				if (myRefNum != 0)
					CloseMovieFile(myRefNum);
				
				// play the movie once thru
				StartMovie(myMovie);
				while (!IsMovieDone(myMovie)  && ContinuePlayMove)
					{
					MoviesTask(myMovie, 0);
					PA_Yield();
					}
				DisposeMovie(myMovie); 
				}
			}
		}
	return(myErr);
}



void StopPlayFileWithQuickTime(void)
{
	ContinuePlayMove = 0;
}



void QT_Init(void)
{
	long            result;
	
	QTInitError = Gestalt( gestaltQuickTime, &result );
	if ( QTInitError == noErr )
		QTInitError = EnterMovies ( );
}

void QT_DeInit(void)
{
	long            result;

	QTInitError = Gestalt( gestaltQuickTime, &result );
	if ( QTInitError == noErr )
		ExitMovies();
}


long QT_SetMovie (PA_PluginParameters params)
{
	long Pfad_len;
	void *Area;
	MovieStuff*			gMovieStuff;
	long err = 0;
	
	Area = (void *) PA_GetLongParameter( params, 1 );	
	if (!Area)
		return -1;
	gMovieStuff = (MovieStuff*) Area;	
	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!
	if(!QTInitError)
		{	

			if ( gMovieStuff->IsExternalAreaInForm == kIsExternalArea )
			{
				if (gMovieStuff->filename != NULL)
					free(gMovieStuff->filename);
				Pfad_len = PA_GetTextParameter( params, 2, 0L );
				gMovieStuff->filename = (char*)malloc(Pfad_len);
				if ( gMovieStuff->filename != NULL )   {
					Pfad_len = PA_GetTextParameter( params, 2, gMovieStuff->filename );
					gMovieStuff->DoJob = 1; 		// open in next on update, then the Mac/Altura Grafport is correct to use for QuickTime
				}
				else
					return -108;
			}
			else
			{
				char		*fullPath;
				long		pathLen				= 0;
				FSSpec		myFSSpec;
				short		movieResFile		= 0;
				fullPath = 0L;

				pathLen = PA_GetTextParameter ( params , 2 , 0L );
				if ( pathLen > 0 )
				{
					fullPath = (char*)malloc(pathLen);
					pathLen = PA_GetTextParameter ( params , 2 , fullPath );
					if ( fullPath != 0L )
					{
						err = build_FSSpecFrompath(fullPath, &myFSSpec);
						
						if ( err == noErr )
						{
							err = OpenMovieFile(&myFSSpec, &movieResFile, fsRdPerm);
							if ( err == noErr )
							{
								// Load movie
								err = NewMovieFromFile(&gMovieStuff->theMovie,movieResFile, nil, nil, newMovieActive, nil);
								CloseMovieFile(movieResFile);
								gMovieStuff->movieOpened = true;
							}
						}	
					}
					else
						err = kErr_MemError;

				}
				else
					err = kErr_InvalidPath;

				if ( fullPath != NULL )
					free ( fullPath );			
			}						
		}
	else
		return QTInitError;

	return err;
}

void QTArea_Execution(PA_PluginParameters params)
{
	MovieStuff*			gMovieStuff;

	gMovieStuff = (MovieStuff*) PA_GetAreaReference( params );
	if (!gMovieStuff)
		return;

	if (gMovieStuff->Tag != 'TMQT')
		return;  // not our area!

	if (gMovieStuff->DoJob > 0)
	{	if (gMovieStuff->DoJob == 2)
			gMovieStuff->DoJob = 0;
		PA_RequestRedraw( params);
	}
	else
		PA_DontTakeEvent( params );
}


long QT_DoMovieJob(PA_PluginParameters params)
{
	ComponentResult		err;
	FSSpec				sfFile;
	short				movieResFile;
	MovieStuff*			gMovieStuff;
    Size				outSize=0;
	Rect		rect;

	gMovieStuff = (MovieStuff*) PA_GetAreaReference( params );
	if (!gMovieStuff)
		return -1;

	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!

	/*
		1 = Set new Movie
		2 = RequestRedraw
		3 = Redraw On Idle
	*/
	
	switch (gMovieStuff->DoJob)
	{
		case 1:   //  Set New Movie
			if (gMovieStuff->theMC)
				DisposeMovieController(gMovieStuff->theMC);
			if (gMovieStuff->theMovie)
				DisposeMovie(gMovieStuff->theMovie);
				
			gMovieStuff->theMovie = NULL;
			gMovieStuff->theMC = NULL;
			
			
			gMovieStuff->DoJob = 0;
			if (gMovieStuff->filename)
				err = build_FSSpecFrompath(gMovieStuff->filename, &sfFile);	
			else
				err = -3;
			if (err == noErr)
				gMovieStuff->IsMovie = !QTUtils_IsImageFile(&sfFile);		
#if VERSIONWIN
			MacSetPort((GrafPtr)GetHWNDPort( (void *)gMovieStuff->FormWindow ));
#endif
			if (err == noErr)
				err = OpenMovieFile(&sfFile, &movieResFile, fsRdPerm);
			if (err == noErr)
			{
				err = NewMovieFromFile(&gMovieStuff->theMovie,movieResFile, nil, nil, newMovieActive, nil);
				CloseMovieFile(movieResFile);
				GetMovieBox( gMovieStuff->theMovie, &rect );
				MacOffsetRect( &rect, -rect.left, -rect.top );
				SetMovieBox( gMovieStuff->theMovie, &rect );
				
				if (err == noErr)
				{
					PA_Rect myrect = PA_GetAreaRect (params);
					
#if VERSIONWIN
					gMovieStuff->theMC = NewMovieController(gMovieStuff->theMovie, (const Rect *) &myrect, mcScaleMovieToFit );
#else
					err = CreateMovieControl (gMovieStuff->FormWindow, (Rect *) &myrect, gMovieStuff->theMovie, kMovieControlOptionLocateTopLeft, &gMovieStuff->MovieControl);

					SizeControl(gMovieStuff->MovieControl, (myrect.fRight-myrect.fLeft), (myrect.fBottom-myrect.fTop));
					err = GetControlData(gMovieStuff->MovieControl, kControlEntireControl,
												kMovieControlDataMovieController, sizeof(gMovieStuff->theMC),
											&gMovieStuff->theMC, &outSize);			
#endif

					// MCEnableEditing(gMovieStuff->theMC, true);
					if(!gMovieStuff->IsMovie)
						err = MCSetVisible (gMovieStuff->theMC, false );
					
					gMovieStuff->movieOpened = TRUE;
					return 0;
				}
			}	

			return -3;
			break;

		default:
			// resize ?
			PA_Rect myrect = PA_GetAreaRect (params);
			rect = *(Rect*)&myrect;
			if (	( rect.left != gMovieStuff->CurMovieBox.left ) ||
					( rect.right != gMovieStuff->CurMovieBox.right ) ||
					( rect.top != gMovieStuff->CurMovieBox.top ) ||
					( rect.bottom != gMovieStuff->CurMovieBox.bottom )  )
				QTSetMovieBox(gMovieStuff, rect);

#if VERSIONWIN
			if ((gMovieStuff->theMovie) && (gMovieStuff->theMC)) {
				ComponentResult err = MCDraw(gMovieStuff->theMC, (WindowRef)GetNativeWindowPort((void *)gMovieStuff->FormWindow));
				// now picture is shown - we have to copy it into offscreen/double buffer

				// testweise schwarzes Viereck malen, sollte anderen Redraw geben
				RECT        WINrect;
				HDC hdc = (HDC) PA_GetUpdateHDC();
				MyPA_RectToRECT(myrect, &WINrect);
				FillRect(hdc, &WINrect, (HBRUSH) GetStockObject(WHITE_BRUSH));

				// Bitmap von Quicktime holen und in DC malen
				HDC hdc4D = (HDC) PA_GetUpdateHDC();
				HDC hdcQT = GetDC(gMovieStuff->FormWindow );
				Boolean success = BitBlt(hdc4D, // handle to destination DC
					rect.left, // x-coord of destination upper-left corner
					rect.top,  // y-coord of destination upper-left corner
					rect.right-rect.left,  // width of destination rectangle
					rect.bottom-rect.top, // height of destination rectangle
					hdcQT,  // handle to source DC
					rect.left,   // x-coordinate of source upper-left corner
					rect.top,   // y-coordinate of source upper-left corner
					SRCCOPY  // raster operation code
					);

			}
#else
			DrawControlInCurrentPort(gMovieStuff->MovieControl);
#endif
			break;

		}
	return 0;
}

long QT_GetScreenshot(void * Area, PA_Handle * PictureShot)
{
	MovieStuff*			gMovieStuff;
	long err = 0;
	
	if (!Area)
		return -1;

	gMovieStuff = (MovieStuff*) Area;

	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!

	if (!gMovieStuff->movieOpened)
		return -3; // movie not open yet

	if (!gMovieStuff->theMC)
		return -4; // controller not available
	
	PicHandle Screenshot = GetMoviePict(gMovieStuff->theMovie, GetMovieTime (gMovieStuff->theMovie, nil));
	// Picture RECT needs byte swapping
	(*Screenshot)->picFrame.bottom = EndianU16_NtoB((*Screenshot)->picFrame.bottom);
	(*Screenshot)->picFrame.top = EndianU16_NtoB((*Screenshot)->picFrame.top);
	(*Screenshot)->picFrame.right = EndianU16_NtoB((*Screenshot)->picFrame.right);
	(*Screenshot)->picFrame.left = EndianU16_NtoB((*Screenshot)->picFrame.left);

	long thePictSize = GetHandleSize((Handle) Screenshot);
	PA_Handle piccopy = PA_NewHandle(thePictSize);
	if (piccopy)
		PA_MoveBlock(*Screenshot, *piccopy, thePictSize);
	else
		err = -108;

	*PictureShot = piccopy; 
	KillPicture(Screenshot);
	return err; 
}


void QTSetMovieBox(MovieStuff*	gMovieStuff, Rect TheAreaRect)
{
	Rect newRect, OrigRect;
	float ZoomX, ZoomY, Zoom;
	RgnHandle    theClip;

	if(gMovieStuff->DisplaySize < -1)
		gMovieStuff->DisplaySize = 0;

	switch(gMovieStuff->DisplaySize)
	{
	case 0:  // default, scale to fit
		newRect = TheAreaRect;
		break;
	case -1: // scale prop
		GetMovieNaturalBoundsRect (gMovieStuff->theMovie, &OrigRect );
		ZoomX = (float) (TheAreaRect.right - TheAreaRect.left) / (float) (OrigRect.right - OrigRect.left);
		ZoomY = (float) (TheAreaRect.bottom - TheAreaRect.top) / (float) (OrigRect.bottom - OrigRect.top);
		Zoom = min(ZoomX, ZoomY);
		newRect = TheAreaRect;
		newRect.right = newRect.left + (short) ((OrigRect.right - OrigRect.left) * Zoom);
		newRect.bottom = newRect.top + (short) ((OrigRect.bottom - OrigRect.top) * Zoom);
		break;
	default: // scale percentage
		GetMovieNaturalBoundsRect (gMovieStuff->theMovie, &OrigRect );
		ZoomX = (float) (TheAreaRect.right - TheAreaRect.left) / (float) (OrigRect.right - OrigRect.left);
		ZoomY = (float) (TheAreaRect.bottom - TheAreaRect.top) / (float) (OrigRect.bottom - OrigRect.top);
		Zoom = min(ZoomX, ZoomY);
		Zoom = min(Zoom, ((float) gMovieStuff->DisplaySize / 100));
		newRect = TheAreaRect;
		newRect.right = newRect.left + (short) ((float) (OrigRect.right - OrigRect.left) * Zoom);
		newRect.bottom = newRect.top + (short) ((float) (OrigRect.bottom - OrigRect.top) * Zoom);
		break;
	}
	
	theClip = NewRgn();
	RectRgn (theClip, &TheAreaRect);
	
#if VERSIONWIN
	SetMovieBox (gMovieStuff->theMovie , &newRect );
	SetMovieDisplayClipRgn (gMovieStuff->theMovie, theClip);
	MCSetControllerBoundsRect (gMovieStuff->theMC, &newRect);
#else
//	SetMovieBox (gMovieStuff->theMovie , &newRect );
//	SetMovieClipRgn (gMovieStuff->theMovie, theClip);
	MoveControl(gMovieStuff->MovieControl, newRect.left, newRect.top);
	SizeControl(gMovieStuff->MovieControl, (newRect.right-newRect.left), (newRect.bottom-newRect.top));
	SetMovieDisplayClipRgn (gMovieStuff->theMovie, theClip);
#endif

	DisposeRgn(theClip);



 gMovieStuff->CurMovieBox = TheAreaRect;

}

long QT_DoScroll(PA_PluginParameters params)
{
	MovieStuff*			gMovieStuff;
	
	gMovieStuff = (MovieStuff*) PA_GetAreaReference( params );
	if (!gMovieStuff)
		return -1;
	
	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!
	
	PA_Rect myrect = PA_GetAreaRect (params);
#if VERSIONWIN
	if ((gMovieStuff->theMovie) && (gMovieStuff->theMC)) {
		ComponentResult err = MCDraw(gMovieStuff->theMC, (WindowRef)GetNativeWindowPort((void *)gMovieStuff->FormWindow));
		}
#else
	MoveControl(gMovieStuff->MovieControl, myrect.fLeft, myrect.fTop);
#endif

	return 0;
}	

long QT_DoIdle(PA_PluginParameters params)
{
#if VERSIONWIN
	MovieStuff*			gMovieStuff;
	ComponentResult err;
	
	gMovieStuff = (MovieStuff*) PA_GetAreaReference( params );
	if (!gMovieStuff)
		return -1;
	
	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!
	
	if(!gMovieStuff->theMC)
		return 0;

	err = MCIdle (gMovieStuff->theMC);
#endif

	return 0;
}	

long QT_DoClick(PA_PluginParameters params)
{
	MovieStuff*			gMovieStuff;
	ComponentResult err;
	PA_Event* event;
	Point  where;
	
	gMovieStuff = (MovieStuff*) PA_GetAreaReference( params );
	if (!gMovieStuff)
		return -1;
	
	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!
	
	event = ( (PA_Event**) params->fParameters )[ 0 ];	
	where.v = event->fWhereV;
	where.h = event->fWhereH;	
#if VERSIONWIN
	err= MCClick (gMovieStuff->theMC, (WindowRef)GetNativeWindowPort((void *)gMovieStuff->FormWindow), where, event->fWhen, event->fModifiers);
#else
	err= MCClick (gMovieStuff->theMC, gMovieStuff->FormWindow, where, event->fWhen, event->fModifiers);

#endif
	return 0;
}
	
long QT_SetControl (void * Area, long command, long value)
{
	MovieStuff*			gMovieStuff;
	ComponentResult err=0;
	long	Actionlong;
	float   frate;
	TimeRecord	SelectionTimeRecord;
	
	if (!Area)
		return -1;

	gMovieStuff = (MovieStuff*) Area;

	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!

	if (!gMovieStuff->movieOpened)
		return -3; // movie not open yet

	if (!gMovieStuff->theMC)
		return -4; // controller not availabe

	switch (command)
	{
		case QTController: 
			err = MCSetVisible (gMovieStuff->theMC, (value == 0));
			gMovieStuff->CurMovieBox.right = 0;  // to check zoom size, may be different with/without controller
			break;

		case QTVolumeIcon: 
			Actionlong = 0;
			err = MCDoAction (gMovieStuff->theMC, mcActionGetFlags, &Actionlong );	
			if (value)
				Actionlong = Actionlong | mcFlagSuppressSpeakerButton;
			else
				Actionlong = Actionlong & ~mcFlagSuppressSpeakerButton;
			err = MCDoAction (gMovieStuff->theMC, mcActionSetFlags, (void *) Actionlong );
			break;

		case QTStepIcon: 
			Actionlong = 0;
			err = MCDoAction (gMovieStuff->theMC, mcActionGetFlags, &Actionlong );	
			if (value)
				Actionlong = Actionlong | mcFlagSuppressStepButtons;
			else
				Actionlong = Actionlong & ~mcFlagSuppressStepButtons;
			err = MCDoAction (gMovieStuff->theMC, mcActionSetFlags, (void *) Actionlong );
			break;

		case QTVolume: 
			err = MCDoAction (gMovieStuff->theMC, mcActionSetVolume, (void *) value );
			break;

		case QTPlay: 
			if (value)
			{	Fixed rate = GetMoviePreferredRate(gMovieStuff->theMovie);
				err = MCDoAction (gMovieStuff->theMC, mcActionPlay, (void *) rate );
			}
			else
				err = MCDoAction (gMovieStuff->theMC, mcActionPlay, (void *) 0);
			break;

		case QTStep: 
			err = MCDoAction (gMovieStuff->theMC, mcActionStep, (void *) value);
			break;

		case QTSpeed: 
			frate = (float) value;
			SetMovieRate (gMovieStuff->theMovie, FloatToFixed (frate / 100));
			break;

		case QTGoto: 
			Actionlong = value;
			if (GetMovieDuration (gMovieStuff->theMovie) < Actionlong)
				Actionlong = GetMovieDuration (gMovieStuff->theMovie);
			SetMovieTimeValue (gMovieStuff->theMovie, Actionlong);  // length in ticks
			break;

		case QTLoop: 
			QTUtils_SetLoopingState (gMovieStuff->theMovie, gMovieStuff->theMC, value);
			break;

		case QTSize: 
			gMovieStuff->DisplaySize = value;
			gMovieStuff->CurMovieBox.right = 0;  // this will request a redraw in new size
			break;
			
		case QTSelectionStart:
			SelectionTimeRecord.value.lo = value;
			SelectionTimeRecord.value.hi = 0;
			SelectionTimeRecord.base = 0;
			SelectionTimeRecord.scale =  GetMovieTimeScale (gMovieStuff->theMovie);
			err = MCDoAction (gMovieStuff->theMC, mcActionSetSelectionBegin, &SelectionTimeRecord );
			break;
			
		case QTSelectionLength:
			SelectionTimeRecord.value.lo = value;
			SelectionTimeRecord.value.hi = 0;
			SelectionTimeRecord.base = 0;
			SelectionTimeRecord.scale =  GetMovieTimeScale (gMovieStuff->theMovie);
			err = MCDoAction (gMovieStuff->theMC, mcActionSetSelectionDuration, &SelectionTimeRecord );
			break;
			
		case QTPlaySelection:
			err = MCDoAction (gMovieStuff->theMC, mcActionSetPlaySelection, &value );
			break;
			
		case QTEditSelection:
			err = MCEnableEditing(gMovieStuff->theMC, (Boolean) value);
			break;

		default:
			return -5;  // unknown command
	}
	if (gMovieStuff->DoJob != 1) 
		gMovieStuff->DoJob = 2;  // requests redraw in next End Execution cycle
	return err;
}

long QT_GetControl (void * Area, long command, long * value)
{
	MovieStuff*			gMovieStuff;
	short   curvolume = 0;
	long	result = 0;
	ComponentResult err=0;
	float   frate;
	Rect	OrigRect;
	TimeRecord	SelectionTimeRecord;

//	Fixed fixedRate;
	
	*value = 0;
		
	if (!Area)
		return -1;

	gMovieStuff = (MovieStuff*) Area;

	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!

	if (!gMovieStuff->movieOpened)
		return -3; // movie not open yet

	// if (!gMovieStuff->theMC)
	// 07.04.2006 Louis Thoumin No controller needs if is an offscreen area
	if ( (!gMovieStuff->theMC) && ( gMovieStuff->IsExternalAreaInForm == kIsExternalArea ) )
		return -4; // controller not availabe

	switch (command)
	{
		case QTController: 
			result = !MCGetVisible (gMovieStuff->theMC);
			break;

		case QTVolumeIcon: 
			err = MCDoAction (gMovieStuff->theMC, mcActionGetFlags, &result );	
			result = result & mcFlagSuppressSpeakerButton;
			break;

		case QTStepIcon: 
			err = MCDoAction (gMovieStuff->theMC, mcActionGetFlags, &result );	
			result = result & mcFlagSuppressStepButtons;
			break;

		case QTVolume: 
			err = MCDoAction (gMovieStuff->theMC, mcActionGetVolume, &curvolume );
			result = curvolume;
			break;

		case QTSpeed: 
			frate = FixedToFloat(GetMovieRate (gMovieStuff->theMovie));
			result = (long) (frate * 100);
			break;

		case QTGetLength: 
			result = GetMovieDuration (gMovieStuff->theMovie); 
			break;

		case QTGoto: 
			result = GetMovieTime (gMovieStuff->theMovie, nil); 
			break;

		case QTLoop: 
			err = QTUtils_GetMovieFileLoopingInfo (gMovieStuff->theMovie, &result);
			break;

		case QTSize: 
			result = gMovieStuff->DisplaySize;
			break;
			
		case QTGetTimeScale: 
			result = GetMovieTimeScale (gMovieStuff->theMovie);
			break;
			
		case QTGetMovieWidth:
			GetMovieNaturalBoundsRect (gMovieStuff->theMovie, &OrigRect );
			result = OrigRect.right - OrigRect.left;
			break;
			
		case QTGetMovieHeight:
			GetMovieNaturalBoundsRect (gMovieStuff->theMovie, &OrigRect );
			result = OrigRect.bottom - OrigRect.top;
			break;
			
		case QTSelectionStart:
			SelectionTimeRecord.value.lo = 0;
			SelectionTimeRecord.value.hi = 0;
			SelectionTimeRecord.base = 0;
			SelectionTimeRecord.scale =  nil;
			err = MCDoAction (gMovieStuff->theMC, mcActionGetSelectionBegin, &SelectionTimeRecord );
			if (err==1)  // no idea why it always returns 1 even it works
				err=0;
			result = SelectionTimeRecord.value.lo;
			break;
			
		case QTSelectionLength:
			SelectionTimeRecord.value.lo = 0;
			SelectionTimeRecord.value.hi = 0;
			SelectionTimeRecord.base = 0;
			SelectionTimeRecord.scale =  nil;
			err = MCDoAction (gMovieStuff->theMC, mcActionGetSelectionDuration, &SelectionTimeRecord );
			if (err==1)  // no idea why it always returns 1 even it works
				err=0;
			result = SelectionTimeRecord.value.lo;
			break;
			
		case QTPlaySelection:
			err = MCDoAction (gMovieStuff->theMC, mcActionGetPlaySelection, &result );
			break;
			
		case QTEditSelection:
			result = MCIsEditingEnabled(gMovieStuff->theMC);
			break;
			
		// 08.04.2006 Louis Thoumin Get Control selector added: QTGetMoviePreferredRate & QTGetMoviePreferredVolume
		case QTGetMoviePreferredRate:	
			//http://developer.apple.com/documentation/QuickTime/APIREF/getmoviepreferredrate.htm#//apple_ref/doc/uid/TP40001187-DontLinkChapterID_1-GetMoviePreferredRate
			/*  
			The  movie rate is a 32-bit, fixed-point number. 
			Positive integers indicate forward rates and negative integers indicate reverse rates.
			*/
			frate = FixedToFloat(GetMoviePreferredRate(gMovieStuff->theMovie));
			result = (long) (frate * 100);
			break;
			
		case QTGetMoviePreferredVolume:
			//http://developer.apple.com/documentation/QuickTime/APIREF/getmoviepreferredvolume.htm#//apple_ref/doc/uid/TP40001187-DontLinkChapterID_1-GetMoviePreferredVolume
			/*
			 The preferred volume setting of the movie. 
			 The volume parameter contains a 16-bit, fixed-point number that contains the movie's default volume. 
			 The high-order 8 bits contain the integer part of the value; 
			 the low-order 8 bits contain the fractional part. 
			 Volume values range from 1.0 to 1.0. 
			 Negative values play no sound but preserve the absolute value of the volume setting.
			
			 kFullVolume: full volume (constant value is 1.0). 
			 kNoVolume: no volume (constant value is 0.0). 
			 */
			result = (long)GetMoviePreferredVolume (gMovieStuff->theMovie);
			break;
			
			
		default:
			return -5;  // unknown command
	}
	*value = result;
	// gMovieStuff->DoJob = 2;
	return err;
}

long QT_PageChange(PA_PluginParameters params)
{
	MovieStuff*			gMovieStuff;
	ComponentResult err;
	short CurrentPage, pageTo, pageFrom;
	
	gMovieStuff = (MovieStuff*) PA_GetAreaReference( params );
	if (!gMovieStuff)
		return -1;
	
	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!
	
	CurrentPage = PA_GetCurrentPage();
	PA_GetPageChange( params, &pageFrom, &pageTo );
	
	if ((CurrentPage == pageTo) && (CurrentPage == gMovieStuff->FormPage))
		{
		 SetMovieActive (gMovieStuff->theMovie, true);
		 if(gMovieStuff->IsMovie)
			 err = MCSetVisible (gMovieStuff->theMC, true );
		}
	else
		if ((CurrentPage == pageFrom) && (CurrentPage == gMovieStuff->FormPage))
			{
			 SetMovieActive (gMovieStuff->theMovie, false);
			 err = MCSetVisible (gMovieStuff->theMC, false );
			}		
		
	return 0;
}	


void QTArea_PreInit(PA_PluginParameters params, short kindOfArea)
{
	MovieStuff* gMovieStuff;
	PA_PluginProperties currentPluginProps;	 
	short err = noErr;

	gMovieStuff = 0L;// 05.04.2006 Louis Thoumin
	// bug?? ACI0026601   
	// PA_GetPluginProperties seems not to work in InitArea. So do it here and copy results.
	if ( kindOfArea == kIsExternalArea ) // 05.04.2006 Louis Thoumin Only if an external area
		gMovieStuff = (MovieStuff *) PA_GetAreaReference( params );

	if (!gMovieStuff )
	{
		if ( kindOfArea == kIsExternalArea ) // 05.04.2006 Louis Thoumin Only if an external area
		{	PA_GetPluginProperties(params, &currentPluginProps);
			err = PA_GetLastError();
		}

		if(!err)
		{
			gMovieStuff = (MovieStuff *) malloc(sizeof(MovieStuff));
			if( gMovieStuff )
			{	gMovieStuff->Tag = 'TMQT';
				gMovieStuff->filename=0L;
				gMovieStuff->movieOpened=false;
				gMovieStuff->theMovie=NULL;
				gMovieStuff->theMC = NULL;
				gMovieStuff->DoJob = 0;
				gMovieStuff->DisplaySize = 0;
				#if __NextVersion__
				if ( kindOfArea == kIsExternalArea ) // 05.04.2006 Louis Thoumin Only if an external area
				{
					long propsize = sizeof(MovieProps);
					gMovieStuff->AdvancedProps = (MovieProps *) PA_GetAdvancedProperties(&currentPluginProps, &propsize);
					if (propsize != sizeof(MovieProps))
						gMovieStuff->AdvancedProps = nil;
				}
				#endif
				
				if ( kindOfArea == kIsExternalArea ) // 05.04.2006 Louis Thoumin Only if an external area
				{
					#if VERSIONWIN
						gMovieStuff->FormWindow = (HWND) currentPluginProps.fWinHWND;
						gMovieStuff->fWinHDC = (HDC) currentPluginProps.fWinHDC;
						CreatePortAssociation (gMovieStuff->FormWindow, NULL, kQTMLNoIdleEvents );
					#else
						gMovieStuff->FormWindow = (CWindowPtr) currentPluginProps.fMacWindow;
						gMovieStuff->fMacPort = (CGrafPtr) currentPluginProps.fMacPort;
					#endif
						gMovieStuff->FormPage = currentPluginProps.fPage;
						PA_SetAreaReference( params, gMovieStuff );
				}
				// Save kind of area. Need later to allow some calls and to deinit the area
				gMovieStuff->IsExternalAreaInForm = kindOfArea; // 05.04.2006 Louis Thoumin
			}
		}
	}

}

void QTArea_Init(PA_PluginParameters params, short kindOfArea) // 05.04.2006 Louis Thoumin Added short kindOfArea
{
	MovieStuff* gMovieStuff;
//	short err = 0;
	gMovieStuff = 0L;


	if ( kindOfArea == kIsExternalArea ) // 05.04.2006 Louis Thoumin Only if an external area
		gMovieStuff = (MovieStuff *) PA_GetAreaReference( params );

	if (!gMovieStuff )
	{   //  should be set in any case from preinit - just never know...
		gMovieStuff = (MovieStuff*) malloc(sizeof(MovieStuff));

		gMovieStuff->Tag = 'TMQT';
		gMovieStuff->filename=0L;
		gMovieStuff->movieOpened=false;
		gMovieStuff->theMovie=NULL;
		gMovieStuff->theMC = NULL;
		gMovieStuff->DoJob = 0;
		gMovieStuff->FormPage = -1;
		gMovieStuff->DisplaySize = 0;
		if ( kindOfArea == kIsExternalArea ) // 05.04.2006 Louis Thoumin Only if an external area
		{
			#if VERSIONWIN
				gMovieStuff->FormWindow = (HWND) PA_GetHWND(0L);
				gMovieStuff->fWinHDC = 0L;
				CreatePortAssociation (gMovieStuff->FormWindow, NULL, kQTMLNoIdleEvents );
			#else
				gMovieStuff->FormWindow = 0L;
				gMovieStuff->fMacPort = 0L;
			#endif
		}
		else
		{
			#if VERSIONWIN
				gMovieStuff->FormWindow = 0L;
				gMovieStuff->fWinHDC = 0L;
			#else
				gMovieStuff->FormWindow = 0L;
				gMovieStuff->fMacPort = 0L;
			#endif
		}

		gMovieStuff->FormPage = 1;  // unknown - error

		gMovieStuff->IsExternalAreaInForm = kindOfArea;
		if ( kindOfArea == kIsNotExternalArea ) // 05.04.2006 Louis Thoumin Return Offscreen reference
		{
			PA_SetLongParameter ( params , 1 , (long)gMovieStuff );
			PA_ReturnLong ( params , noErr );
		}

	}
	else
	{
		if ( kindOfArea == kIsNotExternalArea ) // 05.04.2006 Louis Thoumin Return Offscreen reference
			PA_ReturnLong ( params , kErr_MemError );
	}
}


void QTArea_DeInit(void * myPtr)
{
	MovieStuff* gMovieStuff;

	gMovieStuff = (MovieStuff*) myPtr;

	if (gMovieStuff->theMC)
		DisposeMovieController(gMovieStuff->theMC);
	if (gMovieStuff->theMovie)
		DisposeMovie(gMovieStuff->theMovie);	
	if (gMovieStuff->filename)
		free(gMovieStuff->filename);

	#if VERSIONWIN
		DestroyPortAssociation((CWindowPtr) GetHWNDPort(gMovieStuff->FormWindow));
	#endif

	free(gMovieStuff);
}

#if VERSIONWIN
	void MyPA_RectToRECT(PA_Rect areaRect, RECT *rect)
	{
		rect->bottom = areaRect.fBottom;
		rect->left = areaRect.fLeft;
		rect->right = areaRect.fRight;
		rect->top = areaRect.fTop;
	}
#endif

Boolean QTUtils_IsImageFile (FSSpec *theFSSpec)
{	
	GraphicsImportComponent		myImporter = NULL;
	
	GetGraphicsImporterForFile(theFSSpec, &myImporter);
	if (myImporter != NULL)
		CloseComponent(myImporter);
	
	return(myImporter != NULL);
}

OSErr QTUtils_GetMovieFileLoopingInfo (Movie theMovie, long *theLoopInfo)
{
	UserData			myUserData = NULL;
	long				myLoopInfo = kNoLooping;
	OSErr				myErr = paramErr;

	
	// source: http://www.mactech.com/articles/mactech/Vol.16/16.02/Feb00QTToolkit/
	// make sure we've got a movie
	if (theMovie == NULL)
		goto bail;

	// get the movie's user data list
	myUserData = GetMovieUserData(theMovie);
	if (myUserData != NULL) {
		myErr = GetUserDataItem(myUserData, &myLoopInfo, 
						sizeof(myLoopInfo), FOUR_CHAR_CODE('LOOP'), 0);
		if (myErr == noErr)
			myLoopInfo = EndianS32_BtoN(myLoopInfo);
		if (myErr == userDataItemNotFound)
		{
			myErr = 0;
			myLoopInfo = 2;
		}
	}

bail:
	*theLoopInfo = myLoopInfo;

	return(myErr);
}

void QTUtils_SetLoopingState (Movie theMovie, MovieController theMC, long 	myLoopInfo)
{

	switch (myLoopInfo) {

		case kNormalLooping:
			MCDoAction(theMC, mcActionSetLooping, (void *)true);
			MCDoAction(theMC, mcActionSetLoopIsPalindrome, (void *)false);
			break;

		case kPalindromeLooping:
			MCDoAction(theMC, mcActionSetLooping, (void *)true);
			MCDoAction(theMC, mcActionSetLoopIsPalindrome, (void *)true);
			break;

		case kNoLooping:
		default:
			MCDoAction(theMC, mcActionSetLooping, (void *)false);
			MCDoAction(theMC, mcActionSetLoopIsPalindrome, (void *)false);
			break;
	}
}

#if __NextVersion__
void DesignEditButton(PA_PluginParameters params)
{
   PA_Dial4D      dialog;
   char      lastVarName[256];
   char      okOrCancel;
	PA_PluginProperties currentPluginProps;	 
	MovieProps		*CurAdvancedProps;
	short err;
	long propsize;


		PA_GetPluginProperties(params, &currentPluginProps);
		err = PA_GetLastError();
			if(!err)
			{  CurAdvancedProps = (MovieProps *) PA_GetAdvancedProperties(&currentPluginProps, &propsize);
				if ((!CurAdvancedProps) || (propsize != sizeof(MovieProps)))
				{	CurAdvancedProps = (MovieProps *) malloc(sizeof(MovieProps));
					CurAdvancedProps->AllowEdit = 0;
					CurAdvancedProps->DisplaySize = 0;
					CurAdvancedProps->loop = 2;
					CurAdvancedProps->HideController = 0;
					CurAdvancedProps->HideVolume = 0;
					CurAdvancedProps->HideStep = 0;
					CurAdvancedProps->PlaySelection = 0;
				}
			}
	 

 dialog = PA_NewDialog();
   PA_OpenDialog( dialog, "AdvancedProperties", 0 );
   PA_Dial4DBeginUpdateVariables(dialog);
   switch (CurAdvancedProps->loop)
   {
	   case 2: PA_Dial4DSetLong( dialog, "rloop1", 1 );
		break;
	   case 1: PA_Dial4DSetLong( dialog, "rloop2", 1 );
		break;
		default: 
			PA_Dial4DSetLong( dialog, "rloop3", 1 );
   }
   
   PA_Dial4DEndUpdateVariables(dialog);


// Loop until the user validates/cancels the dialog
   do {
      okOrCancel = PA_ModalDialog( dialog, lastVarName );
   } while ( okOrCancel == 0 );

   // if user validates the dialog, read the picture
   // radio button variables to get the selected value
   if ( okOrCancel == 1 )
   {
      // Look which radio button is 1
      if ( PA_Dial4DGetLong( dialog, "rloop1" ) == 1 )
         CurAdvancedProps->loop = 2; 
      else if ( PA_Dial4DGetLong ( dialog, "rloop2" ) == 1 )
         CurAdvancedProps->loop = 1; 
      else if ( PA_Dial4DGetLong ( dialog, "rloop3" ) == 1 )
         CurAdvancedProps->loop = 0; 
   }
   PA_CloseDialog( dialog );

   PA_SetAdvancedProperties (params, CurAdvancedProps, sizeof(MovieProps));
}
#endif



long QT_GetID3Tags (void * Area, PA_Variable * TagID, PA_Variable * TagContent)
{
  MovieStuff*			gMovieStuff;
  OSType udType;
  short count, i;
  char nul = 0;
  char tag[4];
  UserData inUserData = 0;
  Handle hData = NULL;
  Ptr p;
  OSErr err = noErr;
  long loopcounter = 0;
  // code from http://developer.apple.com/qa/qa2001/qa1135.html
 
	if (!Area)
		return -1;

	gMovieStuff = (MovieStuff*) Area;
	if (gMovieStuff->Tag != 'TMQT')
		return -2;  // not our area!
	if (!gMovieStuff->movieOpened)
		return -3; // movie not open yet

  if ((PA_GetVariableKind(*TagID) != eVK_ArrayText) || (PA_GetVariableKind(*TagContent) != eVK_ArrayText))
		return -5;

  hData = NewHandle(0);
  inUserData = GetMovieUserData(gMovieStuff->theMovie);
  if (!inUserData)
	  return -4;

  PA_ResizeArray (TagID, 0);
  PA_ResizeArray (TagContent, 0);
  udType = GetNextUserDataType(inUserData, 0);
  if(0 != udType) {
    do {
      count = CountUserDataType(inUserData, udType); 
      for(i = 1; i <= count; i++) {
        if((udType>>24) == kUserDataIsText) {
 
          // if the first letter of udType is 0xA9 (the copyright symbol),
          // then use GetUserDataText instead of GetUserData
          // See Movies.h for a list of interesting user data types
          err = GetUserDataText(inUserData, hData, udType, i, 0);  // 0 = langEnglish, we hard code it.
          if (err) goto bail;

          // null-terminate the string in the handle
          PtrAndHand(&nul, hData, 1);

          // turn any CRs into spaces
          p = *hData;
          while(*p) {
            if (*p == kReturnCharCode) *p = ' ';
            p++;
          };

          HLock(hData);
		  PA_ResizeArray (TagID, ++loopcounter);
		  PA_ResizeArray (TagContent, loopcounter);
		  tag[0] = (char)(udType>>24);
		  tag[1] = (char)(udType>>16);
		  tag[2] = (char)(udType>>8);
		  tag[3] = (char)(udType);

		  PA_SetTextInArray (*TagID, loopcounter, (char *) &tag, 4);
		  PA_SetTextInArray (*TagContent, loopcounter, *hData, strlen(*hData));
          HUnlock(hData);

        } else {

          err = GetUserData(inUserData, hData, udType, i);
          if (err) goto bail;

         /* printf("  %c%c%c%c: [%d bytes]\n",
                (char)(udType>>24),
                (char)(udType>>16),
                (char)(udType>>8),
                (char)udType, GetHandleSize(hData));  */

          /*  Content may be numeric, like x/y size, exposure, etc.
		  would need correct handling
		  HLock(hData);
		  PA_ResizeArray (TagID, ++loopcounter);
		  PA_ResizeArray (TagContent, loopcounter);
		  PA_SetTextInArray (*TagID, loopcounter, (char *) &udType, 4);
		  PA_SetTextInArray (*TagContent, loopcounter, *hData, GetHandleSize(hData));
          HUnlock(hData);  */

        }
      }

      udType = GetNextUserDataType(inUserData, udType); 

    } while(0 != udType);
  }

bail:
  err = 0;  //  finished

  DisposeHandle(hData);
  return err;
}

